import pygame, random, math
from globals import *

class Ball(pygame.sprite.Sprite):
	WIDTH  = 8
	HEIGHT = 8
	def createSurface(self, colour, width = 8, height = 8):
		val = pygame.Surface((width, height))
		val.fill([255,0,255])
		val.set_colorkey([255,0,255])
		pygame.draw.circle(val, colour, (width / 2, height / 2), width / 2)
		
		return val
		
	def __init__(self, surf, pos, width = 8, height = 8):
		self.pos = pos
		self.image = surf
		self.rect = pygame.Rect(pos, (width, height))
		pygame.sprite.Sprite.__init__(self)
	
	def update(self):
		self.rect.centerx = self.pos[0]
		self.rect.centery = self.pos[1]
	
	def moveTo(self, pos):
		self.pos = pos
	
	def move(self, delta):
		self.pos = (self.pos[0] + delta[0], self.pos[1] + delta[1])

class GameBall(Ball):
	ATTRACT = 0
	SHOOT   = 1
	RAIN    = 3
	VELOCITY = 7
	SHOT_VELOCITY = 10
	def __init__(self, group, surf, pos, attractor, state = 0, width = 8, height = 8):
		Ball.__init__(self,surf, pos, width, height)
		self.attractor = attractor
		self.state = state
		if state == GameBall.ATTRACT:
			group.add(self)
		self.ball_group = group

	def shooting(self):
		return (self.state == GameBall.SHOOT)
		
	def update(self):
		if (self.state == GameBall.ATTRACT):
			self.ball_group.remove(self)
			if (not pygame.sprite.spritecollideany(self, self.ball_group)):
				dest = self.attractor.pos			
				dist = (dest[0] - self.pos[0], dest[1] - self.pos[1])
				angle = math.atan2(dist[1], dist[0])

				velocity = 3
			else:
				angle = random.randint(0, 360)
				velocity = 3
			self.ball_group.add(self)
			
			self.pos = (self.pos[0] + (math.cos(angle) * GameBall.VELOCITY),
			            self.pos[1] + (math.sin(angle) * GameBall.VELOCITY))
		else:
			self.move((0, self.yvel))
			if (self.pos[1] < -Ball.HEIGHT) or (self.pos[1] > (Global.SCREEN_H + Ball.HEIGHT)):
				if (self.state == GameBall.RAIN):
					Global.player_rain.remove(self)
					Global.enemy_rain.remove(self)
				else:
					self.ball_group.remove(self)
			else:
				if (self.state == GameBall.RAIN) and (pygame.sprite.spritecollideany(self, self.ball_group)):
					self.collect_snd.play()
					self.state = GameBall.ATTRACT
					if self.yvel < 0:
						self.yvel = GameBall.SHOT_VELOCITY
					else:
						self.yvel = -GameBall.SHOT_VELOCITY
					Global.player_rain.remove(self)
					Global.enemy_rain.remove(self)
					self.ball_group.add(self)
			
		Ball.update(self)
		
	def shoot(self):
		self.state = GameBall.SHOOT
		self.shoot_snd.play()
	
class PlayerBall(GameBall):
	surface = None
	def __init__(self, group, pos, attractor, state = 0):
		if (PlayerBall.surface == None):
			PlayerBall.surface = self.createSurface([255, 255, 255])
			PlayerBall.shoot_snd = pygame.mixer.Sound('byew.wav')
			PlayerBall.collect_snd = pygame.mixer.Sound('bloop.wav')
		self.shoot_snd = PlayerBall.shoot_snd
		self.collect_snd = PlayerBall.collect_snd
		GameBall.__init__(self, group, PlayerBall.surface, pos, attractor, state)
		self.yvel = -GameBall.SHOT_VELOCITY
		if state == GameBall.RAIN:
			self.yvel = GameBall.VELOCITY
		self.type = Global.PLAYER


class EnemyBall(GameBall):
	surface = None
	def __init__(self, group, pos, attractor, state = 0):
		if (EnemyBall.surface == None):
			EnemyBall.surface = self.createSurface([0, 0, 0])
			EnemyBall.shoot_snd = pygame.mixer.Sound('bwow.wav')
			EnemyBall.collect_snd = pygame.mixer.Sound('blarp.wav')
		self.shoot_snd = EnemyBall.shoot_snd
		self.collect_snd = EnemyBall.collect_snd
		GameBall.__init__(self, group, EnemyBall.surface, pos, attractor, state)
		self.yvel = GameBall.SHOT_VELOCITY
		if state == GameBall.RAIN:
			self.yvel = -GameBall.VELOCITY
		self.type = Global.ENEMY

class GlowyBall(Ball):
	def __init__(self, pos, type, ball_group, num_balls):
		self.type = type
		self.colour = type
		self.colourdirection = 3
		self.updateImage()
		Ball.__init__(self,self.image, pos, 12, 12)
		ball_group.add(self)
		self.ball_group = ball_group
				
		for i in range(num_balls):
			self.spawn()
		
	
	def shoot(self):
		# get a random ball
		balls = self.ball_group.sprites()
		for ball in balls:
			if (ball != self) and (not ball.shooting()):
				ball.moveTo(self.pos)
				ball.shoot()
				break
				
	def spawn(self):
		if (self.type == Global.PLAYER):
			PlayerBall(self.ball_group, self.pos, self)
		else:
			EnemyBall(self.ball_group, self.pos, self)
			
	def updateImage(self):
		self.image = self.createSurface([self.colour,self.colour,self.colour], 12, 12)
		
	def update(self):
		self.colour = self.colour + self.colourdirection
		if (self.colour <= 0):
			self.colourdirection = 15
			self.colour = 0
		if (self.colour >= 255):
			self.colourdirection = -15
			self.colour = 255
			
		self.updateImage()
		Ball.update(self)
