import pygame
from pygame.locals import *
from ball import *
from globals import *

class Instructions:
	def __init__(self):
		self.bmp = pygame.image.load('instructions.png')
		self.snd = pygame.mixer.Sound('instructions.ogg')
		self.debounce = True
		
	def play(self):
		Global.screen.blit(self.bmp, (0,0))
		pygame.display.flip()
		self.snd.play()
		
		done = False
		while not done:
			pygame.time.delay(50)
			event_pump()
			keys = pygame.key.get_pressed()
			if not self.debounce:
				if keys[K_SPACE] or keys[K_RETURN] or keys[K_ESCAPE]:
					done = True
			else:
				if (not keys[K_SPACE]) and (not keys[K_RETURN]) and (not keys[K_ESCAPE]):
					self.debounce = False
		self.snd.stop()	


class MenuBar(pygame.sprite.Sprite):
	def __init__(self, name, max, ypos, value = 0):
		pygame.sprite.Sprite.__init__(self)
		font = pygame.font.Font('prstartk.ttf', 18)
		self.name = font.render(name, True, [0,0,0])
		self.max = max
		self.image = pygame.Surface((500,60))
		self.image.fill([128,128,128])
		self.image.blit(self.name, (250 - (self.name.get_width() / 2),10))
		self.set_value(value)
		self.rect = pygame.Rect(((Global.SCREEN_W / 2) - 250, ypos), (500,600))
	
	def set_value(self, value):
		if value >= 0 and value <= self.max:
			self.value = value
			pygame.draw.rect(self.image, [0,0,0], pygame.Rect(0,40,500,20))
			colour = 255 - int((float(value) / self.max) * 255)
			pygame.draw.rect(self.image, [255, colour, 0], pygame.Rect(1, 41, int((float(value) / self.max) * 498), 18))
	
	def get_value(self):
		return self.value

	def inc(self):
		self.set_value(self.value + 1)

	def dec(self):
		self.set_value(self.value - 1)
		
class CustomMenu:
	def __init__(self):
		self.background = pygame.image.load('title.png')
		self.selector_group = pygame.sprite.RenderUpdates()
		self.selector = GlowyBall((50, 250), Global.PLAYER, self.selector_group, 15)
		self.playerbar = MenuBar('player blobs', 150, 200, 30)
		self.enemybar = MenuBar('enemy blobs', 150, 280, 30)
		self.all = [self.selector_group, pygame.sprite.RenderUpdates([self.playerbar, self.enemybar])]
		self.selected = self.playerbar
		self.keypressed = True
		self.move_snd = pygame.mixer.Sound('byew.wav')
		self.select_snd = pygame.mixer.Sound('bwow.wav')

	def play(self):
		Global.screen.blit(self.background, (0, 0))
		pygame.display.flip()

		play = False
		while Global.state != Global.GAME_END:
			event_pump()
			
			keys = pygame.key.get_pressed()
			if not self.keypressed:
				if keys[K_UP] or keys[K_DOWN]:
					if self.selected == self.playerbar:
						self.selected = self.enemybar
					else:
						self.selected = self.playerbar
					self.keypressed = True
				if self.keypressed:
					self.move_snd.play()
				if keys[K_RETURN] or keys[K_SPACE]:
					self.select_snd.play()
					play = True
					break
				if keys[K_ESCAPE]:
					play = False
					break
			else:
				if (not keys[K_UP]) and (not keys[K_DOWN]) and (not keys[K_RETURN]) and (not keys[K_SPACE]):
					self.keypressed = False
			if keys[K_RIGHT]:
				self.selected.inc()
			if keys[K_LEFT]:
				self.selected.dec()
			
			if self.selected == self.playerbar:
				self.selector.moveTo((50, 250))
			else:
				self.selector.moveTo((50, 330))
			
			startTime = pygame.time.get_ticks()
	
			rects = []
			for group in self.all:
				group.clear(Global.screen, self.background)
			for group in self.all:
				group.update()
				rects.extend(group.draw(Global.screen))
			pygame.display.update(rects)
	
			frameTime = pygame.time.get_ticks() - startTime
	
			if (frameTime < 40):
				pygame.time.delay(40 - frameTime)
		if play:
			return (self.playerbar.get_value(), self.enemybar.get_value())
		return None
		
class Menu:
	def __init__(self):
		self.background = pygame.image.load('title.png')
		self.selector_group = pygame.sprite.RenderUpdates()
		self.selector = GlowyBall((150, 210), Global.PLAYER, self.selector_group, 15)
		font = pygame.font.Font('prstartk.ttf', 18)
		self.eternal = font.render('eternal mode', True, [0, 0, 0])
		self.custom = font.render('custom mode', True, [0, 0, 0])
		self.instructions = font.render('instructions', True, [0, 0, 0])
		self.quit = font.render('quit', True, [0, 0, 0])
		self.options = [(self.eternal, self.sel_eternal),
						(self.custom, self.sel_custom),
						(self.instructions, self.sel_instructions),
						(self.quit, self.sel_quit)]
		i = 0
		for opt in self.options:
			self.background.blit(opt[0], ((Global.SCREEN_W / 2)- (opt[0].get_width() / 2), 200 + (i * 40)))
			i = i + 1
		self.selected = 0
		self.all = [self.selector_group]
		self.enemies = 30
		self.players = 30
		self.keypressed = True
		self.move_snd = pygame.mixer.Sound('byew.wav')
		self.select_snd = pygame.mixer.Sound('bwow.wav')
		
	def sel_quit(self):
		Global.state = Global.GAME_END
	
	def sel_custom(self):
		result = CustomMenu().play()
		if result != None:
			self.players = result[0]
			self.enemies = result[1]
			Global.state = Global.GAME_PLAY
		else:
			self.keypressed = True
			Global.screen.blit(self.background, (0, 0))
			pygame.display.flip()
			
	def sel_eternal(self):
		Global.state = Global.GAME_PLAY
		self.enemies = 30
		self.players = 30
	
	def sel_instructions(self):
		Instructions().play()
		self.keypressed = True
		Global.screen.blit(self.background, (0, 0))
		pygame.display.flip()

	def play(self):
		Global.screen.blit(self.background, (0, 0))
		pygame.display.flip()
		while Global.state == Global.GAME_MENU:
			event_pump()
			
			keys = pygame.key.get_pressed()
			if not self.keypressed:
				if keys[K_UP]:
					self.selected = self.selected - 1
					if self.selected < 0:
						self.selected = len(self.options) - 1
					self.keypressed = True
				if keys[K_DOWN]:
					self.selected = self.selected + 1
					if self.selected >= len(self.options):
						self.selected = 0
					self.keypressed = True
				if self.keypressed:
					self.move_snd.play()
				if keys[K_RETURN] or keys[K_SPACE]:
					self.select_snd.play()
					self.options[self.selected][1]()
				if keys[K_ESCAPE]:
					self.selected = len(self.options) - 1
			else:
				if (not keys[K_UP]) and (not keys[K_DOWN]) and (not keys[K_RETURN]) and (not keys[K_SPACE]):
					self.keypressed = False
			
			self.selector.moveTo((150, (self.selected * 40) + 210))
			
			startTime = pygame.time.get_ticks()
	
			rects = []
			for group in self.all:
				group.clear(Global.screen, self.background)
			for group in self.all:
				group.update()
				rects.extend(group.draw(Global.screen))
			pygame.display.update(rects)
	
			frameTime = pygame.time.get_ticks() - startTime
	
			if (frameTime < 40):
				pygame.time.delay(40 - frameTime)
		return (self.players, self.enemies)